"""Hook plugin that translates Phlo events into OTel spans and metrics."""

from __future__ import annotations

import hashlib
from datetime import datetime
from typing import Any

from opentelemetry._logs import LogRecord, SeverityNumber
from opentelemetry.context import Context
from opentelemetry.trace import (
    NonRecordingSpan,
    Span,
    SpanContext,
    Status,
    StatusCode,
    TraceFlags,
    TraceState,
    get_current_span,
    set_span_in_context,
)

from phlo.hooks.events import DataMigrationEvent
from phlo.hooks.events import HookCorrelation
from phlo.hooks.events import LogEvent
from phlo.hooks.events import LineageEvent
from phlo.hooks.events import PublishEvent
from phlo.hooks.events import QualityResultEvent
from phlo.hooks.events import SchemaMigrationEvent
from phlo.hooks.events import ServiceLifecycleEvent
from phlo.hooks.events import TelemetryEvent
from phlo.hooks.events import TransformEvent
from phlo.hooks.events import IngestionEvent
from phlo.plugins.base import PluginMetadata
from phlo.plugins.hooks import HookPlugin, HookRegistration

from phlo_otel.provider import get_log_emitter, get_meter, get_tracer


class OtelHookPlugin(HookPlugin):
    """Emit OTel traces and metrics from Phlo hook events."""

    def __init__(self) -> None:
        self._counter_cache: dict[tuple[str, str], Any] = {}
        self._gauge_cache: dict[tuple[str, str], Any] = {}
        self._histogram_cache: dict[tuple[str, str], Any] = {}
        self._up_down_counter_cache: dict[tuple[str, str], Any] = {}

    @property
    def metadata(self) -> PluginMetadata:
        return PluginMetadata(
            name="otel",
            version="0.1.0",
            description="OpenTelemetry instrumentation for Phlo",
        )

    def get_hooks(self) -> list[HookRegistration]:
        return [
            HookRegistration(
                hook_name="otel_ingestion",
                handler=self._handle_ingestion,
            ),
            HookRegistration(
                hook_name="otel_transform",
                handler=self._handle_transform,
            ),
            HookRegistration(
                hook_name="otel_quality",
                handler=self._handle_quality,
            ),
            HookRegistration(
                hook_name="otel_lineage",
                handler=self._handle_lineage,
            ),
            HookRegistration(
                hook_name="otel_publish",
                handler=self._handle_publish,
            ),
            HookRegistration(
                hook_name="otel_service_lifecycle",
                handler=self._handle_service_lifecycle,
            ),
            HookRegistration(
                hook_name="otel_schema_migration",
                handler=self._handle_schema_migration,
            ),
            HookRegistration(
                hook_name="otel_data_migration",
                handler=self._handle_data_migration,
            ),
            HookRegistration(
                hook_name="otel_telemetry",
                handler=self._handle_telemetry,
            ),
            HookRegistration(
                hook_name="otel_log_record",
                handler=self._handle_log_record,
            ),
        ]

    def _handle_ingestion(self, event: Any) -> None:
        if not isinstance(event, IngestionEvent):
            return

        correlation = self._merge_correlation(
            event.correlation,
            run_id=event.run_id,
            asset_key=event.asset_key,
            partition_key=event.partition_key,
        )
        tracer = get_tracer()
        with tracer.start_as_current_span(
            f"ingestion.{event.table_name}",
            attributes={
                "phlo.asset_key": event.asset_key,
                "phlo.stage": "ingestion",
                "phlo.table_name": event.table_name,
                "phlo.group_name": event.group_name,
                "phlo.event_type": event.event_type,
            },
            context=self._build_parent_context(correlation),
        ) as span:
            self._set_correlation_attributes(span, correlation)
            self._set_attribute_if_present(span, "phlo.system", event.tags.get("source"))
            if event.status:
                span.set_attribute("phlo.status", event.status)
            if event.error:
                span.set_status(Status(status_code=StatusCode.ERROR, description=event.error))
            if event.metrics:
                for key, value in event.metrics.items():
                    if isinstance(value, (int, float)):
                        span.set_attribute(f"phlo.metrics.{key}", value)
            self._set_event_tags(span, event.tags)
            self._record_failure(event_name="ingestion", status=event.status, error=event.error)

        counter = self._get_counter(
            "phlo.ingestion.runs",
            description="Number of ingestion runs",
        )
        counter.add(1, {"table_name": event.table_name, "status": event.status or "unknown"})

        rows = event.metrics.get("rows_loaded")
        if rows is None:
            rows = event.metrics.get("rows_processed")
        if isinstance(rows, (int, float)):
            rows_counter = self._get_counter(
                "phlo.ingestion.rows",
                description="Rows ingested",
            )
            rows_counter.add(int(rows), {"table_name": event.table_name})

        self._record_duration(
            "phlo.ingestion.duration",
            metrics=event.metrics,
            attributes={"table_name": event.table_name, "status": event.status or "unknown"},
            description="Ingestion duration",
        )

    def _handle_transform(self, event: Any) -> None:
        if not isinstance(event, TransformEvent):
            return

        correlation = self._merge_correlation(
            event.correlation,
            run_id=getattr(event, "run_id", None),
            asset_key=event.asset_key,
            partition_key=event.partition_key,
        )
        tracer = get_tracer()
        span_name = f"transform.{event.tool}"
        if event.target:
            span_name = f"transform.{event.tool}.{event.target}"

        with tracer.start_as_current_span(
            span_name,
            attributes={
                "phlo.stage": "transform",
                "phlo.system": event.tool,
                "phlo.tool": event.tool,
                "phlo.event_type": event.event_type,
            },
            context=self._build_parent_context(correlation),
        ) as span:
            self._set_correlation_attributes(span, correlation)
            if event.target:
                span.set_attribute("phlo.target", event.target)
            if event.model_names:
                span.set_attribute("phlo.model_names", event.model_names)
            if event.status:
                span.set_attribute("phlo.status", event.status)
            if event.error:
                span.set_status(Status(status_code=StatusCode.ERROR, description=event.error))
            self._set_numeric_span_attributes(span, event.metrics)
            self._set_event_tags(span, event.tags)
            self._record_failure(event_name="transform", status=event.status, error=event.error)

        counter = self._get_counter(
            "phlo.transform.runs",
            description="Number of transform runs",
        )
        counter.add(1, {"tool": event.tool, "status": event.status or "unknown"})

        self._record_duration(
            "phlo.transform.duration",
            metrics=event.metrics,
            attributes={"tool": event.tool, "status": event.status or "unknown"},
            description="Transform duration",
        )

    def _handle_quality(self, event: Any) -> None:
        if not isinstance(event, QualityResultEvent):
            return

        correlation = self._merge_correlation(
            event.correlation,
            run_id=getattr(event, "run_id", None),
            asset_key=event.asset_key,
            partition_key=event.partition_key,
            check_name=event.check_name,
        )
        tracer = get_tracer()
        with tracer.start_as_current_span(
            f"quality.{event.check_name}",
            attributes={
                "phlo.asset_key": event.asset_key,
                "phlo.stage": "quality",
                "phlo.check_name": event.check_name,
                "phlo.passed": event.passed,
                "phlo.event_type": event.event_type,
            },
            context=self._build_parent_context(correlation),
        ) as span:
            self._set_correlation_attributes(span, correlation)
            self._set_attribute_if_present(span, "phlo.system", event.tags.get("backend"))
            self._set_attribute_if_present(span, "phlo.operation", event.check_type)
            if event.severity:
                span.set_attribute("phlo.severity", event.severity)
            if event.check_type:
                span.set_attribute("phlo.check_type", event.check_type)
            for key, value in event.metadata.items():
                if isinstance(value, (bool, int, float, str)):
                    span.set_attribute(f"phlo.metadata.{key}", value)
            if not event.passed:
                span.set_status(
                    Status(
                        status_code=StatusCode.ERROR,
                        description=f"Quality check failed: {event.check_name}",
                    )
                )
                self._record_failure(event_name="quality", status="fail")
            self._set_event_tags(span, event.tags)

        counter = self._get_counter(
            "phlo.quality.checks",
            description="Quality check executions",
        )
        result = "pass" if event.passed else "fail"
        counter.add(1, {"check_name": event.check_name, "result": result})

    def _handle_lineage(self, event: Any) -> None:
        if not isinstance(event, LineageEvent):
            return

        correlation = self._merge_correlation(event.correlation)
        tracer = get_tracer()
        with tracer.start_as_current_span(
            "lineage.edges",
            attributes={
                "phlo.event_type": event.event_type,
                "phlo.stage": "lineage",
                "phlo.operation": "edges",
                "phlo.edge_count": len(event.edges),
                "phlo.asset_count": len(event.asset_keys),
            },
            context=self._build_parent_context(correlation),
        ) as span:
            self._set_correlation_attributes(span, correlation)
            if event.asset_keys:
                span.set_attribute("phlo.asset_keys", sorted(event.asset_keys))
            self._set_event_tags(span, event.tags)
            for key, value in event.metadata.items():
                if isinstance(value, (bool, int, float, str)):
                    span.set_attribute(f"phlo.metadata.{key}", value)

        event_counter = self._get_counter(
            "phlo.lineage.events",
            description="Number of lineage events",
        )
        edge_counter = self._get_counter(
            "phlo.lineage.edges",
            description="Number of lineage edges",
        )
        attributes = self._metric_attributes_from_tags(event.tags)
        event_counter.add(1, attributes)
        edge_counter.add(len(event.edges), attributes)

    def _handle_publish(self, event: Any) -> None:
        if not isinstance(event, PublishEvent):
            return

        correlation = self._merge_correlation(
            event.correlation,
            run_id=getattr(event, "run_id", None),
            asset_key=event.asset_key,
            partition_key=getattr(event, "partition_key", None),
        )
        target_system = event.target_system or "unknown"
        tracer = get_tracer()
        with tracer.start_as_current_span(
            f"publish.{target_system}",
            attributes={
                "phlo.event_type": event.event_type,
                "phlo.stage": "publish",
                "phlo.system": target_system,
                "phlo.operation": "publish",
                "phlo.target_system": target_system,
            },
            context=self._build_parent_context(correlation),
        ) as span:
            self._set_correlation_attributes(span, correlation)
            if event.status:
                span.set_attribute("phlo.status", event.status)
            if event.tables:
                span.set_attribute("phlo.table_count", len(event.tables))
                span.set_attribute("phlo.tables", sorted(event.tables.values()))
            if event.error:
                span.set_status(Status(status_code=StatusCode.ERROR, description=event.error))
            self._set_numeric_span_attributes(span, event.metrics)
            self._set_event_tags(span, event.tags)
            self._record_failure(event_name="publish", status=event.status, error=event.error)

        counter = self._get_counter(
            "phlo.publish.runs",
            description="Number of publish runs",
        )
        counter.add(1, {"target_system": target_system, "status": event.status or "unknown"})

        if event.tables:
            table_counter = self._get_counter(
                "phlo.publish.tables",
                description="Number of published tables",
            )
            table_counter.add(len(event.tables), {"target_system": target_system})

        self._record_duration(
            "phlo.publish.duration",
            metrics=event.metrics,
            attributes={"target_system": target_system, "status": event.status or "unknown"},
            description="Publish duration",
        )

    def _handle_service_lifecycle(self, event: Any) -> None:
        if not isinstance(event, ServiceLifecycleEvent):
            return

        correlation = self._merge_correlation(event.correlation)
        phase = event.phase or "unknown"
        tracer = get_tracer()
        with tracer.start_as_current_span(
            f"service.{event.service_name}.{phase}",
            attributes={
                "phlo.event_type": event.event_type,
                "phlo.stage": "service",
                "phlo.system": "service",
                "phlo.operation": phase,
                "phlo.service_name": event.service_name,
                "phlo.phase": phase,
            },
            context=self._build_parent_context(correlation),
        ) as span:
            self._set_correlation_attributes(span, correlation)
            if event.project_name:
                span.set_attribute("phlo.project_name", event.project_name)
            if event.project_root:
                span.set_attribute("phlo.project_root", event.project_root)
            if event.container_name:
                span.set_attribute("phlo.container_name", event.container_name)
            if event.status:
                span.set_attribute("phlo.status", event.status)
                if self._is_failure_status(event.status):
                    span.set_status(
                        Status(
                            status_code=StatusCode.ERROR,
                            description=f"Service lifecycle failed: {phase}",
                        )
                    )
            self._set_event_tags(span, event.tags)
            self._record_failure(event_name="service_lifecycle", status=event.status)

        counter = self._get_counter(
            "phlo.service.lifecycle.events",
            description="Number of service lifecycle events",
        )
        counter.add(
            1,
            {
                "service_name": event.service_name,
                "phase": phase,
                "status": event.status or "unknown",
            },
        )

    def _handle_schema_migration(self, event: Any) -> None:
        if not isinstance(event, SchemaMigrationEvent):
            return

        correlation = self._merge_correlation(event.correlation)
        tracer = get_tracer()
        with tracer.start_as_current_span(
            f"schema_migration.{event.table_name}",
            attributes={
                "phlo.event_type": event.event_type,
                "phlo.stage": "migration",
                "phlo.system": "schema",
                "phlo.operation": "schema_migration",
                "phlo.table_name": event.table_name,
                "phlo.classification": event.classification,
                "phlo.change_count": event.change_count,
                "phlo.status": event.status,
            },
            context=self._build_parent_context(correlation),
        ) as span:
            self._set_correlation_attributes(span, correlation)
            if event.changes:
                span.set_attribute("phlo.schema_change_count", len(event.changes))
            self._set_event_tags(span, event.tags)
            if self._is_failure_status(event.status):
                span.set_status(
                    Status(
                        status_code=StatusCode.ERROR,
                        description=f"Schema migration failed: {event.table_name}",
                    )
                )
            self._record_failure(event_name="schema_migration", status=event.status)

        labels = {"classification": event.classification, "status": event.status}
        counter = self._get_counter(
            "phlo.schema_migration.runs",
            description="Number of schema migration events",
        )
        counter.add(1, labels)

        changes_counter = self._get_counter(
            "phlo.schema_migration.changes",
            description="Schema change count",
        )
        changes_counter.add(event.change_count, labels)

    def _handle_data_migration(self, event: Any) -> None:
        if not isinstance(event, DataMigrationEvent):
            return

        correlation = self._merge_correlation(
            event.correlation,
            run_id=getattr(event, "run_id", None),
        )
        tracer = get_tracer()
        with tracer.start_as_current_span(
            f"data_migration.{event.migration_name}",
            attributes={
                "phlo.event_type": event.event_type,
                "phlo.stage": "migration",
                "phlo.system": event.source_type,
                "phlo.operation": "data_migration",
                "phlo.migration_name": event.migration_name,
                "phlo.source_type": event.source_type,
                "phlo.destination_table": event.destination_table,
                "phlo.rows_read": event.rows_read,
                "phlo.rows_written": event.rows_written,
                "phlo.status": event.status,
            },
            context=self._build_parent_context(correlation),
        ) as span:
            self._set_correlation_attributes(span, correlation)
            if event.chunk_index is not None:
                span.set_attribute("phlo.chunk_index", event.chunk_index)
            self._set_numeric_span_attributes(span, event.metrics)
            self._set_event_tags(span, event.tags)
            if self._is_failure_status(event.status):
                span.set_status(
                    Status(
                        status_code=StatusCode.ERROR,
                        description=f"Data migration failed: {event.migration_name}",
                    )
                )
            self._record_failure(event_name="data_migration", status=event.status)

        labels = {"source_type": event.source_type, "status": event.status}
        runs_counter = self._get_counter(
            "phlo.data_migration.runs",
            description="Number of data migration events",
        )
        runs_counter.add(1, labels)

        rows_read_counter = self._get_counter(
            "phlo.data_migration.rows_read",
            description="Rows read during data migration",
        )
        rows_read_counter.add(event.rows_read, labels)

        rows_written_counter = self._get_counter(
            "phlo.data_migration.rows_written",
            description="Rows written during data migration",
        )
        rows_written_counter.add(event.rows_written, labels)

        self._record_duration(
            "phlo.data_migration.duration",
            metrics=event.metrics,
            attributes=labels,
            description="Data migration duration",
        )

    def _handle_telemetry(self, event: Any) -> None:
        if not isinstance(event, TelemetryEvent):
            return
        if event.event_type != "telemetry.metric":
            return
        if not isinstance(event.value, (int, float)):
            return
        if self._handle_maintenance_telemetry(event):
            return

        metric_name = f"phlo.telemetry.{event.name}"
        metric_kind = self._resolve_metric_kind(event.payload)
        attributes = self._normalize_metric_attributes(event.payload)
        unit = event.unit or ""

        if metric_kind == "counter":
            counter = self._get_counter(
                metric_name,
                description=f"Telemetry metric: {event.name}",
                unit=unit,
            )
            counter.add(event.value, attributes)
            return

        if metric_kind == "histogram":
            histogram = self._get_histogram(
                metric_name,
                unit=unit,
                description=f"Telemetry metric: {event.name}",
            )
            histogram.record(event.value, attributes)
            return

        if metric_kind == "up_down_counter":
            counter = self._get_up_down_counter(
                metric_name,
                unit=unit,
                description=f"Telemetry metric: {event.name}",
            )
            counter.add(event.value, attributes)
            return

        gauge = self._get_gauge(
            metric_name,
            unit=unit,
            description=f"Telemetry metric: {event.name}",
        )
        gauge.set(event.value, attributes)

    def _handle_maintenance_telemetry(self, event: TelemetryEvent) -> bool:
        if not event.name.startswith("iceberg.maintenance."):
            return False

        attributes = {
            **self._metric_attributes_from_tags(event.tags),
            **self._normalize_metric_attributes(event.payload),
        }
        metric_map: dict[str, tuple[str, str]] = {
            "iceberg.maintenance.run": ("counter", "phlo.maintenance.runs"),
            "iceberg.maintenance.duration_seconds": ("histogram", "phlo.maintenance.duration"),
            "iceberg.maintenance.tables_processed": (
                "counter",
                "phlo.maintenance.tables_processed",
            ),
            "iceberg.maintenance.errors": ("counter", "phlo.maintenance.errors"),
            "iceberg.maintenance.snapshots_deleted": (
                "counter",
                "phlo.maintenance.snapshots_deleted",
            ),
            "iceberg.maintenance.orphan_files": ("counter", "phlo.maintenance.orphan_files"),
            "iceberg.maintenance.total_records": ("counter", "phlo.maintenance.records_processed"),
            "iceberg.maintenance.total_size_mb": ("histogram", "phlo.maintenance.size_mb"),
        }
        mapping = metric_map.get(event.name)
        if mapping is None:
            return False

        metric_type, metric_name = mapping
        if metric_type == "counter":
            counter = self._get_counter(
                metric_name,
                description=f"Maintenance metric derived from {event.name}",
                unit=event.unit or "",
            )
            counter.add(event.value, attributes)
            return True

        histogram = self._get_histogram(
            metric_name,
            description=f"Maintenance metric derived from {event.name}",
            unit=event.unit or "",
        )
        histogram.record(event.value, attributes)
        return True

    def _handle_log_record(self, event: Any) -> None:
        if not isinstance(event, LogEvent):
            return

        attributes = self._build_log_attributes(event)
        trace_id, span_id, trace_flags = self._resolve_log_context(event)
        log_record = LogRecord(
            timestamp=self._datetime_to_unix_nanos(event.timestamp),
            observed_timestamp=self._datetime_to_unix_nanos(datetime.now(event.timestamp.tzinfo)),
            trace_id=trace_id,
            span_id=span_id,
            trace_flags=trace_flags,
            severity_text=event.level.upper(),
            severity_number=self._map_severity(event.level),
            body=event.message,
            attributes=attributes,
        )
        emitter = get_log_emitter()
        if emitter is None:
            return
        emitter.emit(log_record)

    def _get_counter(self, name: str, *, description: str, unit: str = "") -> Any:
        cache_key = (name, unit)
        counter = self._counter_cache.get(cache_key)
        if counter is None:
            counter = get_meter().create_counter(name, unit=unit, description=description)
            self._counter_cache[cache_key] = counter
        return counter

    def _get_gauge(self, name: str, *, unit: str, description: str) -> Any:
        cache_key = (name, unit)
        gauge = self._gauge_cache.get(cache_key)
        if gauge is None:
            gauge = get_meter().create_gauge(name, unit=unit, description=description)
            self._gauge_cache[cache_key] = gauge
        return gauge

    def _get_histogram(self, name: str, *, unit: str, description: str) -> Any:
        cache_key = (name, unit)
        histogram = self._histogram_cache.get(cache_key)
        if histogram is None:
            histogram = get_meter().create_histogram(name, unit=unit, description=description)
            self._histogram_cache[cache_key] = histogram
        return histogram

    def _get_up_down_counter(self, name: str, *, unit: str, description: str) -> Any:
        cache_key = (name, unit)
        counter = self._up_down_counter_cache.get(cache_key)
        if counter is None:
            counter = get_meter().create_up_down_counter(name, unit=unit, description=description)
            self._up_down_counter_cache[cache_key] = counter
        return counter

    def _resolve_metric_kind(self, payload: dict[str, Any]) -> str:
        """Resolve telemetry metric type from reserved payload attributes."""
        raw_kind = payload.get("metric_kind", payload.get("otel_metric_kind", "gauge"))
        if not isinstance(raw_kind, str):
            return "gauge"

        metric_kind = raw_kind.strip().lower().replace("-", "_")
        valid_kinds = {"counter", "gauge", "histogram", "up_down_counter"}
        if metric_kind not in valid_kinds:
            return "gauge"
        return metric_kind

    def _normalize_metric_attributes(self, payload: dict[str, Any]) -> dict[str, Any]:
        normalized: dict[str, Any] = {}
        for key, value in payload.items():
            if key not in self._allowed_metric_label_keys():
                continue
            normalized[key] = self._coerce_metric_attribute_value(value)
        return normalized

    def _set_numeric_span_attributes(self, span: Span, metrics: dict[str, Any]) -> None:
        for key, value in metrics.items():
            if isinstance(value, (int, float)):
                span.set_attribute(f"phlo.metrics.{key}", value)

    def _set_event_tags(self, span: Span, tags: dict[str, str]) -> None:
        for key, value in tags.items():
            span.set_attribute(f"phlo.tags.{key}", value)

    def _set_correlation_attributes(self, span: Span, correlation: HookCorrelation) -> None:
        if correlation.request_id:
            span.set_attribute("phlo.request_id", correlation.request_id)
        if correlation.run_id:
            span.set_attribute("phlo.run_id", correlation.run_id)
        if correlation.asset_key:
            span.set_attribute("phlo.asset_key", correlation.asset_key)
        if correlation.job_name:
            span.set_attribute("phlo.job_name", correlation.job_name)
        if correlation.partition_key:
            span.set_attribute("phlo.partition_key", correlation.partition_key)
        if correlation.check_name:
            span.set_attribute("phlo.check_name", correlation.check_name)

    def _set_attribute_if_present(self, span: Span, key: str, value: str | None) -> None:
        if value:
            span.set_attribute(key, value)

    def _metric_attributes_from_tags(self, tags: dict[str, str]) -> dict[str, str]:
        return {
            key: value for key, value in tags.items() if key in self._allowed_metric_label_keys()
        }

    def _allowed_metric_label_keys(self) -> set[str]:
        return {
            "backend",
            "classification",
            "environment",
            "namespace",
            "operation",
            "phase",
            "result",
            "service",
            "source",
            "source_type",
            "status",
            "target",
            "target_system",
            "tool",
        }

    def _build_log_attributes(self, event: LogEvent) -> dict[str, Any]:
        correlation = self._merge_correlation(
            event.correlation,
            run_id=event.run_id,
            asset_key=event.asset_key,
            job_name=event.job_name,
            partition_key=event.partition_key,
            check_name=event.check_name,
        )
        attributes: dict[str, Any] = {
            "phlo.event_type": event.event_type,
            "phlo.stage": self._event_stage(event.event_type),
            "phlo.logger": event.logger,
            "phlo.level": event.level,
        }
        if event.service:
            attributes["phlo.service"] = event.service
            attributes["phlo.system"] = event.service
        operation = event.tags.get("operation")
        if operation:
            attributes["phlo.operation"] = operation
        if correlation.request_id:
            attributes["phlo.request_id"] = correlation.request_id
        if correlation.run_id:
            attributes["phlo.run_id"] = correlation.run_id
        if correlation.asset_key:
            attributes["phlo.asset_key"] = correlation.asset_key
        if correlation.job_name:
            attributes["phlo.job_name"] = correlation.job_name
        if correlation.partition_key:
            attributes["phlo.partition_key"] = correlation.partition_key
        if correlation.check_name:
            attributes["phlo.check_name"] = correlation.check_name

        for key, value in event.tags.items():
            attributes[f"phlo.tag.{key}"] = value
        for key, value in event.metadata.items():
            attributes[f"phlo.metadata.{key}"] = self._coerce_attribute_value(value)
        return attributes

    def _coerce_attribute_value(self, value: Any) -> Any:
        if isinstance(value, (bool, int, float, str)):
            return value
        if isinstance(value, (list, tuple)):
            return [
                item if isinstance(item, (bool, int, float, str)) else str(item) for item in value
            ]
        return str(value)

    def _coerce_metric_attribute_value(self, value: Any) -> Any:
        if isinstance(value, (bool, int, float, str)):
            return value
        if isinstance(value, (list, tuple)):
            return tuple(
                item if isinstance(item, (bool, int, float, str)) else str(item) for item in value
            )
        return str(value)

    def _map_severity(self, level: str) -> SeverityNumber:
        normalized = level.strip().lower()
        if normalized == "debug":
            return SeverityNumber.DEBUG
        if normalized in {"warning", "warn"}:
            return SeverityNumber.WARN
        if normalized == "error":
            return SeverityNumber.ERROR
        if normalized == "critical":
            return SeverityNumber.FATAL
        return SeverityNumber.INFO

    def _event_stage(self, event_type: str) -> str:
        prefix = event_type.split(".", maxsplit=1)[0]
        if prefix in {"schema_migration", "data_migration"}:
            return "migration"
        if prefix == "service":
            return "service"
        if prefix == "telemetry":
            return "telemetry"
        if prefix == "log":
            return "log"
        return prefix

    def _is_failure_status(self, status: str) -> bool:
        return status.lower() in {"error", "failed", "failure", "rejected"}

    def _record_duration(
        self,
        metric_name: str,
        *,
        metrics: dict[str, Any],
        attributes: dict[str, str],
        description: str,
    ) -> None:
        duration = self._extract_duration_seconds(metrics)
        if duration is None:
            return

        histogram = self._get_histogram(metric_name, unit="s", description=description)
        histogram.record(duration, attributes)

    def _extract_duration_seconds(self, metrics: dict[str, Any]) -> float | None:
        for key in ("duration_seconds", "total_elapsed_seconds", "elapsed_seconds"):
            value = metrics.get(key)
            if isinstance(value, (int, float)):
                return float(value)
        return None

    def _record_failure(
        self,
        *,
        event_name: str,
        status: str | None,
        error: str | None = None,
    ) -> None:
        if not (error or (status and self._is_failure_status(status))):
            return

        counter = self._get_counter(
            "phlo.errors",
            description="Number of failed Phlo workflow events",
        )
        counter.add(1, {"event": event_name, "status": status or "error"})

    def _datetime_to_unix_nanos(self, value: datetime) -> int:
        return int(value.timestamp() * 1_000_000_000)

    def _resolve_log_context(self, event: LogEvent) -> tuple[int | None, int | None, Any | None]:
        correlation = self._merge_correlation(
            event.correlation,
            trace_id=event.metadata.get("trace_id"),
            span_id=event.metadata.get("span_id"),
            trace_flags=event.metadata.get("trace_flags"),
            run_id=event.run_id,
            asset_key=event.asset_key,
            job_name=event.job_name,
            partition_key=event.partition_key,
            check_name=event.check_name,
        )
        span_context = get_current_span().get_span_context()
        trace_id = span_id = trace_flags = None
        if span_context.is_valid:
            trace_id = span_context.trace_id
            span_id = span_context.span_id
            trace_flags = span_context.trace_flags

        metadata_trace_id = self._parse_trace_identifier(correlation.trace_id)
        metadata_span_id = self._parse_trace_identifier(correlation.span_id)
        metadata_trace_flags = self._parse_trace_flags(correlation.trace_flags)
        if metadata_trace_id is not None:
            trace_id = metadata_trace_id
        if metadata_span_id is not None:
            span_id = metadata_span_id
        if metadata_trace_flags is not None:
            trace_flags = metadata_trace_flags
        if trace_id is None or span_id is None:
            synthetic_trace_id, synthetic_span_id = self._derive_trace_context_identifiers(
                correlation
            )
            if trace_id is None:
                trace_id = synthetic_trace_id
            if span_id is None:
                span_id = synthetic_span_id
            if trace_flags is None and synthetic_trace_id is not None:
                trace_flags = TraceFlags(0x01)
        return trace_id, span_id, trace_flags

    def _merge_correlation(self, correlation: HookCorrelation, **overrides: Any) -> HookCorrelation:
        merged = HookCorrelation(**vars(correlation))
        for key, value in overrides.items():
            if value is not None:
                setattr(merged, key, str(value))
        return merged

    def _build_parent_context(self, correlation: HookCorrelation) -> Context | None:
        trace_id = self._parse_trace_identifier(correlation.trace_id)
        span_id = self._parse_trace_identifier(correlation.span_id)
        if trace_id is None or span_id is None:
            synthetic_trace_id, synthetic_span_id = self._derive_trace_context_identifiers(
                correlation
            )
            if trace_id is None:
                trace_id = synthetic_trace_id
            if span_id is None:
                span_id = synthetic_span_id
        if trace_id is None or span_id is None:
            return None

        trace_flags = self._parse_trace_flags(correlation.trace_flags) or TraceFlags(0x01)
        span_context = SpanContext(
            trace_id=trace_id,
            span_id=span_id,
            is_remote=True,
            trace_flags=trace_flags,
            trace_state=TraceState(),
        )
        return set_span_in_context(NonRecordingSpan(span_context))

    def _derive_trace_context_identifiers(
        self,
        correlation: HookCorrelation,
    ) -> tuple[int | None, int | None]:
        grouping_key = self._trace_grouping_key(correlation)
        if grouping_key is None:
            return None, None

        trace_id = self._stable_identifier(f"trace:{grouping_key}", bytes_length=16)
        span_id = self._stable_identifier(f"span:{grouping_key}", bytes_length=8)
        return trace_id, span_id

    def _trace_grouping_key(self, correlation: HookCorrelation) -> str | None:
        if correlation.run_id:
            return f"run:{correlation.run_id}"
        if correlation.request_id:
            return f"request:{correlation.request_id}"
        return None

    def _stable_identifier(self, value: str, *, bytes_length: int) -> int:
        digest = hashlib.sha256(value.encode("utf-8")).digest()
        return int.from_bytes(digest[:bytes_length], byteorder="big", signed=False)

    def _parse_trace_identifier(self, value: Any) -> int | None:
        if value is None:
            return None
        if isinstance(value, int):
            return value
        if not isinstance(value, str):
            return None

        raw = value.strip().lower()
        if not raw:
            return None
        try:
            if raw.startswith("0x"):
                return int(raw, 16)
            return int(raw, 16)
        except ValueError:
            return None

    def _parse_trace_flags(self, value: Any) -> TraceFlags | None:
        trace_flags = self._parse_trace_identifier(value)
        if trace_flags is None:
            return None
        return TraceFlags(trace_flags & 0xFF)



"""OTel TracerProvider and MeterProvider setup.

Configured via standard OTEL_* environment variables:
- OTEL_EXPORTER_OTLP_ENDPOINT (default: http://localhost:4317)
- OTEL_SERVICE_NAME (default: phlo)
- OTEL_SERVICE_NAMESPACE (default: phlo)
- OTEL_SERVICE_VERSION (default: package version)
- OTEL_SERVICE_INSTANCE_ID (default: hostname)
- OTEL_TRACES_EXPORTER (default: unset; set to otlp or configure an OTLP endpoint)
- OTEL_METRICS_EXPORTER (default: unset; set to otlp or configure an OTLP endpoint)
- OTEL_LOGS_EXPORTER (default: none; set to otlp to enable OTLP log export)
- OTEL_RESOURCE_ATTRIBUTES
"""

from __future__ import annotations

import atexit
import importlib
import os
import socket
from typing import Any, cast

from phlo.config import get_settings
from phlo.logging import get_logger

logger = get_logger(__name__)

metrics = cast(Any, importlib.import_module("opentelemetry.metrics"))
trace = cast(Any, importlib.import_module("opentelemetry.trace"))
OTLPMetricExporter = cast(
    Any,
    importlib.import_module("opentelemetry.exporter.otlp.proto.grpc.metric_exporter"),
).OTLPMetricExporter
OTLPLogExporter = cast(
    Any,
    importlib.import_module("opentelemetry.exporter.otlp.proto.grpc._log_exporter"),
).OTLPLogExporter
OTLPSpanExporter = cast(
    Any,
    importlib.import_module("opentelemetry.exporter.otlp.proto.grpc.trace_exporter"),
).OTLPSpanExporter
LoggerProvider = cast(Any, importlib.import_module("opentelemetry.sdk._logs")).LoggerProvider
BatchLogRecordProcessor = cast(
    Any,
    importlib.import_module("opentelemetry.sdk._logs.export"),
).BatchLogRecordProcessor
MeterProvider = cast(
    Any,
    importlib.import_module("opentelemetry.sdk.metrics"),
).MeterProvider
PeriodicExportingMetricReader = cast(
    Any,
    importlib.import_module("opentelemetry.sdk.metrics.export"),
).PeriodicExportingMetricReader
resources = cast(Any, importlib.import_module("opentelemetry.sdk.resources"))
SERVICE_NAME = resources.SERVICE_NAME
Resource = resources.Resource
TracerProvider = cast(Any, importlib.import_module("opentelemetry.sdk.trace")).TracerProvider
BatchSpanProcessor = cast(
    Any,
    importlib.import_module("opentelemetry.sdk.trace.export"),
).BatchSpanProcessor
Meter = Any
Tracer = Any

_initialized = False
_logger_provider: LoggerProvider | None = None
_tracer_provider: TracerProvider | None = None
_meter_provider: MeterProvider | None = None

INSTRUMENTATION_NAME = "phlo"
INSTRUMENTATION_VERSION = "0.1.0"
PACKAGE_NAME = "phlo-otel"


def _build_resource_attributes() -> dict[str, str]:
    """Build OTel resource attributes for Phlo process metadata."""
    settings = get_settings()
    service_name = os.environ.get("OTEL_SERVICE_NAME", settings.phlo_log_service_name)
    service_namespace = os.environ.get("OTEL_SERVICE_NAMESPACE", settings.phlo_service_namespace)
    service_version = os.environ.get(
        "OTEL_SERVICE_VERSION",
        settings.phlo_service_version or INSTRUMENTATION_VERSION,
    )
    service_instance_id = os.environ.get(
        "OTEL_SERVICE_INSTANCE_ID",
        settings.phlo_service_instance_id or socket.gethostname(),
    )
    project = os.environ.get("PHLO_PROJECT", settings.phlo_project or service_name)
    return {
        SERVICE_NAME: service_name,
        "service.namespace": service_namespace,
        "service.version": service_version,
        "service.instance.id": service_instance_id,
        "deployment.environment": settings.phlo_environment,
        "phlo.package": PACKAGE_NAME,
        "phlo.runtime": "python",
        "phlo.project": project,
    }


def _logs_export_enabled() -> bool:
    """Return whether OTLP log export is enabled for this process."""
    return _signal_export_enabled(
        exporter_env="OTEL_LOGS_EXPORTER",
        signal_endpoint_env="OTEL_EXPORTER_OTLP_LOGS_ENDPOINT",
        default_enabled=False,
    )


def _traces_export_enabled() -> bool:
    """Return whether OTLP trace export is enabled for this process."""
    return _signal_export_enabled(
        exporter_env="OTEL_TRACES_EXPORTER",
        signal_endpoint_env="OTEL_EXPORTER_OTLP_TRACES_ENDPOINT",
        default_enabled=False,
    )


def _metrics_export_enabled() -> bool:
    """Return whether OTLP metrics export is enabled for this process."""
    return _signal_export_enabled(
        exporter_env="OTEL_METRICS_EXPORTER",
        signal_endpoint_env="OTEL_EXPORTER_OTLP_METRICS_ENDPOINT",
        default_enabled=False,
    )


def _signal_export_enabled(
    *,
    exporter_env: str,
    signal_endpoint_env: str,
    default_enabled: bool = True,
) -> bool:
    """Return whether an OTLP signal should be exported."""
    exporter = os.environ.get(exporter_env)
    if exporter is not None:
        return exporter.strip().lower() not in {"", "none"}
    if os.environ.get(signal_endpoint_env) or os.environ.get("OTEL_EXPORTER_OTLP_ENDPOINT"):
        return True
    return default_enabled


def _ensure_initialized() -> None:
    """Set up global TracerProvider and MeterProvider once."""
    global _initialized, _logger_provider, _meter_provider, _tracer_provider
    if _initialized:
        return

    resource_attributes = _build_resource_attributes()
    resource = Resource.create(resource_attributes)

    tracer_provider = None
    if _traces_export_enabled():
        tracer_provider = TracerProvider(resource=resource)
        tracer_provider.add_span_processor(BatchSpanProcessor(OTLPSpanExporter()))
        trace.set_tracer_provider(tracer_provider)

    logger_provider = None
    if _logs_export_enabled():
        logger_provider = LoggerProvider(resource=resource)
        logger_provider.add_log_record_processor(BatchLogRecordProcessor(OTLPLogExporter()))

    meter_provider = None
    if _metrics_export_enabled():
        metric_reader = PeriodicExportingMetricReader(OTLPMetricExporter())
        meter_provider = MeterProvider(resource=resource, metric_readers=[metric_reader])
        metrics.set_meter_provider(meter_provider)

    _logger_provider = logger_provider
    _tracer_provider = tracer_provider
    _meter_provider = meter_provider
    _initialized = True
    logger.info(
        "otel_initialized",
        service_name=resource_attributes[SERVICE_NAME],
        service_namespace=resource_attributes["service.namespace"],
        environment=resource_attributes["deployment.environment"],
    )


def get_tracer() -> Tracer:
    """Return the Phlo OTel tracer."""
    if not _traces_export_enabled():
        return trace.get_tracer(INSTRUMENTATION_NAME, INSTRUMENTATION_VERSION)
    _ensure_initialized()
    return trace.get_tracer(INSTRUMENTATION_NAME, INSTRUMENTATION_VERSION)


def get_meter() -> Meter:
    """Return the Phlo OTel meter."""
    if not _metrics_export_enabled():
        return metrics.get_meter(INSTRUMENTATION_NAME, INSTRUMENTATION_VERSION)
    _ensure_initialized()
    return metrics.get_meter(INSTRUMENTATION_NAME, INSTRUMENTATION_VERSION)


def get_log_emitter() -> Any:
    """Return the Phlo OTel log emitter."""
    _ensure_initialized()
    if _logger_provider is None:
        return None
    return _logger_provider.get_logger(INSTRUMENTATION_NAME, INSTRUMENTATION_VERSION)


def shutdown_otel() -> None:
    """Flush and stop OTel providers created by this module."""
    global _initialized, _logger_provider, _meter_provider, _tracer_provider

    for provider in (_logger_provider, _meter_provider, _tracer_provider):
        if provider is None or not _provider_needs_shutdown(provider):
            continue
        shutdown = getattr(provider, "shutdown", None)
        if callable(shutdown):
            shutdown()

    _logger_provider = None
    _meter_provider = None
    _tracer_provider = None
    _initialized = False


def _provider_needs_shutdown(provider: Any) -> bool:
    """Return whether an OTel provider still has active shutdown work."""
    if hasattr(provider, "_shutdown"):
        return not bool(getattr(provider, "_shutdown"))
    if hasattr(provider, "_at_exit_handler"):
        return getattr(provider, "_at_exit_handler") is not None
    if hasattr(provider, "_atexit_handler"):
        return getattr(provider, "_atexit_handler") is not None
    return True


atexit.register(shutdown_otel)



"""Tests for the OTel hook plugin."""

from __future__ import annotations

from unittest.mock import MagicMock, patch

import pytest
from opentelemetry.trace import get_current_span

from phlo.hooks.emitters import TelemetryEventContext, TelemetryEventEmitter
from phlo.hooks.events import (
    DataMigrationEvent,
    HookCorrelation,
    IngestionEvent,
    LineageEvent,
    LogEvent,
    PublishEvent,
    QualityResultEvent,
    SchemaMigrationEvent,
    ServiceLifecycleEvent,
    TelemetryEvent,
    TransformEvent,
)

from phlo_otel.hooks_plugin import OtelHookPlugin


@pytest.fixture()
def plugin() -> OtelHookPlugin:
    return OtelHookPlugin()


@pytest.fixture(autouse=True)
def _mock_otel(monkeypatch):
    """Prevent real OTel provider init; stub tracer and meter."""
    monkeypatch.setattr("phlo_otel.provider._initialized", True)


class TestOtelHookPlugin:
    def test_metadata(self, plugin: OtelHookPlugin):
        assert plugin.metadata.name == "otel"

    def test_registers_otel_hooks(self, plugin: OtelHookPlugin):
        hooks = plugin.get_hooks()
        assert len(hooks) == 10
        names = {h.hook_name for h in hooks}
        assert names == {
            "otel_ingestion",
            "otel_transform",
            "otel_quality",
            "otel_lineage",
            "otel_publish",
            "otel_service_lifecycle",
            "otel_schema_migration",
            "otel_data_migration",
            "otel_telemetry",
            "otel_log_record",
        }

    def test_ignores_wrong_event_type(self, plugin: OtelHookPlugin):
        wrong = TransformEvent(event_type="transform.start", tool="dbt")
        plugin._handle_ingestion(wrong)  # should not raise

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_ingestion_creates_span(self, mock_tracer_fn, mock_meter_fn, plugin):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_counter = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.return_value = mock_counter
        mock_meter_fn.return_value = mock_meter

        event = IngestionEvent(
            event_type="ingestion.complete",
            asset_key="dlt_glucose_entries",
            table_name="glucose_entries",
            group_name="nightscout",
            status="success",
            metrics={"rows_loaded": 500},
        )
        plugin._handle_ingestion(event)

        mock_tracer.start_as_current_span.assert_called_once()
        call_args = mock_tracer.start_as_current_span.call_args
        assert "ingestion.glucose_entries" in call_args[0]
        assert call_args.kwargs["attributes"]["phlo.stage"] == "ingestion"

        assert mock_meter.create_counter.call_count == 2
        assert mock_counter.add.call_count == 2

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_ingestion_uses_correlation_as_parent_context(
        self, mock_tracer_fn, mock_meter_fn, plugin
    ):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_meter = MagicMock()
        mock_meter.create_counter.return_value = MagicMock()
        mock_meter_fn.return_value = mock_meter

        event = IngestionEvent(
            event_type="ingestion.complete",
            asset_key="dlt_glucose_entries",
            table_name="glucose_entries",
            group_name="nightscout",
            run_id="run-123",
            partition_key="2026-03-08",
            correlation=HookCorrelation(
                trace_id="abc123",
                span_id="def456",
                job_name="daily_ingestion",
            ),
        )

        plugin._handle_ingestion(event)

        _, kwargs = mock_tracer.start_as_current_span.call_args
        assert kwargs["context"] is not None
        mock_span.set_attribute.assert_any_call("phlo.run_id", "run-123")
        mock_span.set_attribute.assert_any_call("phlo.partition_key", "2026-03-08")
        mock_span.set_attribute.assert_any_call("phlo.job_name", "daily_ingestion")

    def test_build_parent_context_derives_stable_trace_from_run_id(self, plugin):
        first = plugin._build_parent_context(HookCorrelation(run_id="run-123"))
        second = plugin._build_parent_context(HookCorrelation(run_id="run-123"))

        assert first is not None
        assert second is not None
        first_context = get_current_span(first).get_span_context()
        second_context = get_current_span(second).get_span_context()
        assert first_context.is_valid
        assert second_context.is_valid
        assert first_context.trace_id == second_context.trace_id
        assert first_context.span_id == second_context.span_id

    def test_build_parent_context_uses_request_id_when_run_id_missing(self, plugin):
        context = plugin._build_parent_context(HookCorrelation(request_id="req-99"))

        assert context is not None
        span_context = get_current_span(context).get_span_context()
        assert span_context.is_valid

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_ingestion_preserves_zero_rows_loaded(self, mock_tracer_fn, mock_meter_fn, plugin):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_runs_counter = MagicMock()
        mock_rows_counter = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.side_effect = [mock_runs_counter, mock_rows_counter]
        mock_meter_fn.return_value = mock_meter

        event = IngestionEvent(
            event_type="ingestion.complete",
            asset_key="dlt_glucose_entries",
            table_name="glucose_entries",
            group_name="nightscout",
            status="success",
            metrics={"rows_loaded": 0},
        )

        plugin._handle_ingestion(event)

        mock_rows_counter.add.assert_called_once_with(0, {"table_name": "glucose_entries"})

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_ingestion_records_duration_histogram(self, mock_tracer_fn, mock_meter_fn, plugin):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_runs_counter = MagicMock()
        mock_rows_counter = MagicMock()
        mock_histogram = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.side_effect = [mock_runs_counter, mock_rows_counter]
        mock_meter.create_histogram.return_value = mock_histogram
        mock_meter_fn.return_value = mock_meter

        event = IngestionEvent(
            event_type="ingestion.complete",
            asset_key="dlt_glucose_entries",
            table_name="glucose_entries",
            group_name="nightscout",
            status="success",
            metrics={"rows_loaded": 5, "total_elapsed_seconds": 3.25},
        )

        plugin._handle_ingestion(event)

        mock_histogram.record.assert_called_once_with(
            3.25,
            {"table_name": "glucose_entries", "status": "success"},
        )

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_quality_fail_sets_error_status(self, mock_tracer_fn, mock_meter_fn, plugin):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_meter = MagicMock()
        mock_meter.create_counter.return_value = MagicMock()
        mock_meter_fn.return_value = mock_meter

        event = QualityResultEvent(
            event_type="quality.result",
            asset_key="dlt_glucose_entries",
            check_name="not_null_glucose",
            passed=False,
            severity="error",
        )
        plugin._handle_quality(event)

        mock_span.set_status.assert_called_once()

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_lineage_creates_span_and_counters(self, mock_tracer_fn, mock_meter_fn, plugin):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_events_counter = MagicMock()
        mock_edges_counter = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.side_effect = [mock_events_counter, mock_edges_counter]
        mock_meter_fn.return_value = mock_meter

        event = LineageEvent(
            event_type="lineage.edges",
            edges=[("bronze.orders", "silver.orders"), ("silver.orders", "gold.orders")],
            asset_keys=["silver.orders", "gold.orders"],
            metadata={"pipeline": "orders"},
            tags={"tool": "dbt", "target": "warehouse"},
        )

        plugin._handle_lineage(event)

        mock_tracer.start_as_current_span.assert_called_once_with(
            "lineage.edges",
            attributes={
                "phlo.event_type": "lineage.edges",
                "phlo.stage": "lineage",
                "phlo.operation": "edges",
                "phlo.edge_count": 2,
                "phlo.asset_count": 2,
            },
            context=None,
        )
        mock_events_counter.add.assert_called_once_with(1, {"tool": "dbt", "target": "warehouse"})
        mock_edges_counter.add.assert_called_once_with(2, {"tool": "dbt", "target": "warehouse"})

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_publish_creates_span_and_counters(self, mock_tracer_fn, mock_meter_fn, plugin):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_runs_counter = MagicMock()
        mock_tables_counter = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.side_effect = [mock_runs_counter, mock_tables_counter]
        mock_meter_fn.return_value = mock_meter

        event = PublishEvent(
            event_type="publish.end",
            asset_key="gold.orders",
            target_system="clickhouse",
            tables={"gold.orders": "analytics.orders"},
            status="success",
            metrics={"rows_written": 200},
        )

        plugin._handle_publish(event)

        mock_tracer.start_as_current_span.assert_called_once()
        call_args = mock_tracer.start_as_current_span.call_args
        assert call_args.kwargs["attributes"]["phlo.stage"] == "publish"
        assert call_args.kwargs["attributes"]["phlo.system"] == "clickhouse"
        assert call_args.kwargs["attributes"]["phlo.operation"] == "publish"
        mock_runs_counter.add.assert_called_once_with(
            1, {"target_system": "clickhouse", "status": "success"}
        )
        mock_tables_counter.add.assert_called_once_with(1, {"target_system": "clickhouse"})
        mock_span.set_attribute.assert_any_call("phlo.metrics.rows_written", 200)

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_publish_failure_records_error_and_duration(
        self, mock_tracer_fn, mock_meter_fn, plugin
    ):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_runs_counter = MagicMock()
        mock_error_counter = MagicMock()
        mock_histogram = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.side_effect = [mock_error_counter, mock_runs_counter]
        mock_meter.create_histogram.return_value = mock_histogram
        mock_meter_fn.return_value = mock_meter

        event = PublishEvent(
            event_type="publish.end",
            asset_key="gold.orders",
            target_system="clickhouse",
            status="failure",
            metrics={"elapsed_seconds": 1.75},
            error="warehouse unavailable",
        )

        plugin._handle_publish(event)

        mock_error_counter.add.assert_called_once_with(1, {"event": "publish", "status": "failure"})
        mock_histogram.record.assert_called_once_with(
            1.75,
            {"target_system": "clickhouse", "status": "failure"},
        )

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_service_lifecycle_sets_error_status_on_failure(
        self, mock_tracer_fn, mock_meter_fn, plugin
    ):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        mock_counter = MagicMock()
        mock_error_counter = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.side_effect = [mock_error_counter, mock_counter]
        mock_meter_fn.return_value = mock_meter

        event = ServiceLifecycleEvent(
            event_type="service.post_start",
            service_name="dagster-webserver",
            phase="post_start",
            status="failure",
            tags={"service": "dagster-webserver"},
        )

        plugin._handle_service_lifecycle(event)

        call_args = mock_tracer.start_as_current_span.call_args
        assert call_args.kwargs["attributes"]["phlo.stage"] == "service"
        assert call_args.kwargs["attributes"]["phlo.operation"] == "post_start"
        mock_span.set_status.assert_called_once()
        mock_error_counter.add.assert_called_once_with(
            1,
            {
                "event": "service_lifecycle",
                "status": "failure",
            },
        )
        mock_counter.add.assert_called_once_with(
            1,
            {
                "service_name": "dagster-webserver",
                "phase": "post_start",
                "status": "failure",
            },
        )

    @patch("phlo_otel.hooks_plugin.get_meter")
    @patch("phlo_otel.hooks_plugin.get_tracer")
    def test_schema_and_data_migrations_emit_metrics(self, mock_tracer_fn, mock_meter_fn, plugin):
        mock_span = MagicMock()
        mock_span.__enter__ = MagicMock(return_value=mock_span)
        mock_span.__exit__ = MagicMock(return_value=False)
        mock_tracer = MagicMock()
        mock_tracer.start_as_current_span.return_value = mock_span
        mock_tracer_fn.return_value = mock_tracer

        schema_runs = MagicMock()
        schema_changes = MagicMock()
        data_runs = MagicMock()
        data_rows_read = MagicMock()
        data_rows_written = MagicMock()
        data_duration = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.side_effect = [
            schema_runs,
            schema_changes,
            data_runs,
            data_rows_read,
            data_rows_written,
        ]
        mock_meter.create_histogram.return_value = data_duration
        mock_meter_fn.return_value = mock_meter

        schema_event = SchemaMigrationEvent(
            event_type="schema_migration.applied",
            table_name="silver.orders",
            classification="minor",
            change_count=3,
            status="applied",
        )
        data_event = DataMigrationEvent(
            event_type="data_migration.completed",
            migration_name="backfill_orders",
            source_type="postgres",
            destination_table="silver.orders",
            status="completed",
            rows_read=100,
            rows_written=95,
            chunk_index=2,
            metrics={"duration_seconds": 8.5},
        )

        plugin._handle_schema_migration(schema_event)
        plugin._handle_data_migration(data_event)

        schema_call = mock_tracer.start_as_current_span.call_args_list[0]
        data_call = mock_tracer.start_as_current_span.call_args_list[1]
        assert schema_call.kwargs["attributes"]["phlo.stage"] == "migration"
        assert schema_call.kwargs["attributes"]["phlo.system"] == "schema"
        assert data_call.kwargs["attributes"]["phlo.stage"] == "migration"
        assert data_call.kwargs["attributes"]["phlo.system"] == "postgres"
        schema_changes.add.assert_called_once_with(
            3, {"classification": "minor", "status": "applied"}
        )
        data_rows_read.add.assert_called_once_with(
            100, {"source_type": "postgres", "status": "completed"}
        )
        data_rows_written.add.assert_called_once_with(
            95, {"source_type": "postgres", "status": "completed"}
        )
        data_duration.record.assert_called_once_with(
            8.5, {"source_type": "postgres", "status": "completed"}
        )

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_telemetry_gauge(self, mock_meter_fn, plugin):
        mock_gauge = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_gauge.return_value = mock_gauge
        mock_meter_fn.return_value = mock_meter

        event = TelemetryEvent(
            event_type="telemetry.metric",
            name="ingestion_lag_seconds",
            value=12.5,
            unit="s",
            payload={"source": "nightscout"},
        )
        plugin._handle_telemetry(event)

        mock_meter.create_gauge.assert_called_once()
        mock_gauge.set.assert_called_once_with(12.5, {"source": "nightscout"})

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_standard_instruments_are_cached(self, mock_meter_fn, plugin):
        mock_meter = MagicMock()
        mock_runs_counter = MagicMock()
        mock_rows_counter = MagicMock()
        mock_meter.create_counter.side_effect = [mock_runs_counter, mock_rows_counter]
        mock_meter_fn.return_value = mock_meter

        first = IngestionEvent(
            event_type="ingestion.complete",
            asset_key="dlt_glucose_entries",
            table_name="glucose_entries",
            group_name="nightscout",
            status="success",
            metrics={"rows_loaded": 5},
        )
        second = IngestionEvent(
            event_type="ingestion.complete",
            asset_key="dlt_glucose_entries",
            table_name="glucose_entries",
            group_name="nightscout",
            status="success",
            metrics={"rows_loaded": 7},
        )

        with patch("phlo_otel.hooks_plugin.get_tracer") as mock_tracer_fn:
            mock_span = MagicMock()
            mock_span.__enter__ = MagicMock(return_value=mock_span)
            mock_span.__exit__ = MagicMock(return_value=False)
            mock_tracer = MagicMock()
            mock_tracer.start_as_current_span.return_value = mock_span
            mock_tracer_fn.return_value = mock_tracer

            plugin._handle_ingestion(first)
            plugin._handle_ingestion(second)

        assert mock_meter.create_counter.call_count == 2
        assert mock_runs_counter.add.call_count == 2
        assert mock_rows_counter.add.call_count == 2

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_telemetry_counter_metric_kind(self, mock_meter_fn, plugin):
        mock_counter = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.return_value = mock_counter
        mock_meter_fn.return_value = mock_meter

        event = TelemetryEvent(
            event_type="telemetry.metric",
            name="rows_written",
            value=42,
            unit="rows",
            payload={"metric_kind": "counter", "source": "nightscout"},
        )

        plugin._handle_telemetry(event)

        mock_meter.create_counter.assert_called_once()
        mock_counter.add.assert_called_once_with(42, {"source": "nightscout"})

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_telemetry_histogram_metric_kind(self, mock_meter_fn, plugin):
        mock_histogram = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_histogram.return_value = mock_histogram
        mock_meter_fn.return_value = mock_meter

        event = TelemetryEvent(
            event_type="telemetry.metric",
            name="duration_seconds",
            value=1.5,
            unit="s",
            payload={"otel_metric_kind": "histogram", "source": "nightscout"},
        )

        plugin._handle_telemetry(event)

        mock_meter.create_histogram.assert_called_once()
        mock_histogram.record.assert_called_once_with(1.5, {"source": "nightscout"})

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_telemetry_normalizes_sequence_attributes(self, mock_meter_fn, plugin):
        mock_gauge = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_gauge.return_value = mock_gauge
        mock_meter_fn.return_value = mock_meter

        event = TelemetryEvent(
            event_type="telemetry.metric",
            name="transform.duration_seconds",
            value=2.5,
            unit="seconds",
            payload={"models": ["orders", "customers"]},
        )

        plugin._handle_telemetry(event)

        mock_gauge.set.assert_called_once_with(2.5, {})

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_telemetry_filters_high_cardinality_attributes(self, mock_meter_fn, plugin):
        mock_gauge = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_gauge.return_value = mock_gauge
        mock_meter_fn.return_value = mock_meter

        event = TelemetryEvent(
            event_type="telemetry.metric",
            name="ingestion_lag_seconds",
            value=12.5,
            unit="s",
            payload={"source": "nightscout", "run_id": "run-123", "asset_key": "raw.orders"},
        )

        plugin._handle_telemetry(event)

        mock_gauge.set.assert_called_once_with(12.5, {"source": "nightscout"})

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_telemetry_skips_non_numeric(self, mock_meter_fn, plugin):
        event = TelemetryEvent(
            event_type="telemetry.log",
            name="some_log",
            value="not a number",
        )
        plugin._handle_telemetry(event)
        mock_meter_fn.assert_not_called()

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_telemetry_skips_numeric_logs(self, mock_meter_fn, plugin):
        event = TelemetryEvent(
            event_type="telemetry.log",
            name="some_log",
            value=1,
            payload={"source": "nightscout"},
        )

        plugin._handle_telemetry(event)

        mock_meter_fn.assert_not_called()

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_maintenance_telemetry_promotes_standard_metrics(
        self, mock_meter_fn, monkeypatch, plugin
    ):
        mock_runs_counter = MagicMock()
        mock_duration_histogram = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_counter.return_value = mock_runs_counter
        mock_meter.create_histogram.return_value = mock_duration_histogram
        mock_meter_fn.return_value = mock_meter

        class RecordingBus:
            def __init__(self) -> None:
                self.events: list[object] = []

            def emit(self, event: object) -> None:
                self.events.append(event)

        bus = RecordingBus()
        monkeypatch.setattr("phlo.hooks.emitters.get_hook_bus", lambda: bus)
        telemetry = TelemetryEventEmitter(
            TelemetryEventContext(
                tags={
                    "operation": "expire_snapshots",
                    "namespace": "raw",
                    "status": "success",
                }
            )
        )
        telemetry.emit_metric(
            name="iceberg.maintenance.run",
            value=1,
            unit="run",
            payload={"operation": "expire_snapshots", "namespace": "raw", "status": "success"},
        )
        telemetry.emit_metric(
            name="iceberg.maintenance.duration_seconds",
            value=3.5,
            unit="seconds",
            payload={"operation": "expire_snapshots", "namespace": "raw", "status": "success"},
        )

        for event in bus.events:
            plugin._handle_telemetry(event)

        mock_meter.create_counter.assert_called_once_with(
            "phlo.maintenance.runs",
            unit="run",
            description="Maintenance metric derived from iceberg.maintenance.run",
        )
        mock_runs_counter.add.assert_called_once_with(
            1,
            {"operation": "expire_snapshots", "namespace": "raw", "status": "success"},
        )
        mock_meter.create_histogram.assert_called_once_with(
            "phlo.maintenance.duration",
            unit="seconds",
            description="Maintenance metric derived from iceberg.maintenance.duration_seconds",
        )
        mock_duration_histogram.record.assert_called_once_with(
            3.5,
            {"operation": "expire_snapshots", "namespace": "raw", "status": "success"},
        )

    @patch("phlo_otel.hooks_plugin.get_meter")
    def test_unknown_telemetry_keeps_generic_metric_namespace(self, mock_meter_fn, plugin):
        mock_gauge = MagicMock()
        mock_meter = MagicMock()
        mock_meter.create_gauge.return_value = mock_gauge
        mock_meter_fn.return_value = mock_meter

        event = TelemetryEvent(
            event_type="telemetry.metric",
            name="custom.backlog",
            value=9,
            unit="items",
            payload={"operation": "sync"},
        )

        plugin._handle_telemetry(event)

        mock_meter.create_gauge.assert_called_once_with(
            "phlo.telemetry.custom.backlog",
            unit="items",
            description="Telemetry metric: custom.backlog",
        )
        mock_gauge.set.assert_called_once_with(9, {"operation": "sync"})

    @patch("phlo_otel.hooks_plugin.get_log_emitter")
    def test_log_record_exports_to_otel_logs(self, mock_get_log_emitter, plugin):
        mock_emitter = MagicMock()
        mock_get_log_emitter.return_value = mock_emitter

        event = LogEvent(
            event_type="log.record",
            logger="phlo.tests.logging",
            level="warning",
            message="lag spike detected",
            service="phlo-worker",
            run_id="run-7",
            metadata={"trace_id": "abc123", "attempt": 2},
            tags={"team": "analytics"},
        )

        plugin._handle_log_record(event)

        mock_get_log_emitter.assert_called_once()
        emitted_record = mock_emitter.emit.call_args.args[0]
        assert emitted_record.body == "lag spike detected"
        assert emitted_record.severity_text == "WARNING"
        assert emitted_record.trace_id == int("abc123", 16)
        assert emitted_record.attributes["phlo.stage"] == "log"
        assert emitted_record.attributes["phlo.service"] == "phlo-worker"
        assert emitted_record.attributes["phlo.system"] == "phlo-worker"
        assert emitted_record.attributes["phlo.run_id"] == "run-7"
        assert emitted_record.attributes["phlo.tag.team"] == "analytics"
        assert emitted_record.attributes["phlo.metadata.trace_id"] == "abc123"

    @patch("phlo_otel.hooks_plugin.get_log_emitter")
    def test_log_record_uses_operation_tag_as_semantic_attribute(
        self, mock_get_log_emitter, plugin
    ):
        mock_emitter = MagicMock()
        mock_get_log_emitter.return_value = mock_emitter

        event = LogEvent(
            event_type="log.record",
            logger="phlo.tests.logging",
            level="info",
            message="maintenance complete",
            tags={"operation": "expire_snapshots"},
        )

        plugin._handle_log_record(event)

        emitted_record = mock_emitter.emit.call_args.args[0]
        assert emitted_record.attributes["phlo.operation"] == "expire_snapshots"

    @patch("phlo_otel.hooks_plugin.get_log_emitter")
    def test_log_record_uses_correlation_for_trace_context(self, mock_get_log_emitter, plugin):
        mock_emitter = MagicMock()
        mock_get_log_emitter.return_value = mock_emitter

        event = LogEvent(
            event_type="log.record",
            logger="phlo.tests.logging",
            level="info",
            message="pipeline started",
            correlation=HookCorrelation(
                trace_id="abc123",
                span_id="def456",
                run_id="run-42",
                asset_key="silver.orders",
            ),
        )

        plugin._handle_log_record(event)

        emitted_record = mock_emitter.emit.call_args.args[0]
        assert emitted_record.trace_id == int("abc123", 16)
        assert emitted_record.span_id == int("def456", 16)
        assert emitted_record.attributes["phlo.run_id"] == "run-42"
        assert emitted_record.attributes["phlo.asset_key"] == "silver.orders"

    @patch("phlo_otel.hooks_plugin.get_log_emitter")
    def test_log_record_derives_trace_context_from_run_id(self, mock_get_log_emitter, plugin):
        mock_emitter = MagicMock()
        mock_get_log_emitter.return_value = mock_emitter

        event = LogEvent(
            event_type="log.record",
            logger="phlo.tests.logging",
            level="info",
            message="pipeline heartbeat",
            run_id="run-777",
        )

        plugin._handle_log_record(event)

        emitted_record = mock_emitter.emit.call_args.args[0]
        assert emitted_record.trace_id is not None
        assert emitted_record.span_id is not None
        assert emitted_record.attributes["phlo.run_id"] == "run-777"

    def test_parse_trace_identifier_treats_all_digit_strings_as_hex(self, plugin):
        assert plugin._parse_trace_identifier("1234567890123456") == int("1234567890123456", 16)



"""OpenTelemetry instrumentation for Phlo hook events."""

from phlo_otel.provider import get_log_emitter, get_meter, get_tracer, shutdown_otel

__all__ = [
    "get_log_emitter",
    "get_tracer",
    "get_meter",
    "shutdown_otel",
]



"""Tests for the OTel provider resource configuration."""

from __future__ import annotations

from types import SimpleNamespace

from phlo_otel import provider


def test_build_resource_attributes(monkeypatch):
    monkeypatch.setenv("OTEL_SERVICE_NAME", "phlo-api")
    monkeypatch.setenv("OTEL_SERVICE_NAMESPACE", "phlohouse")
    monkeypatch.setenv("OTEL_SERVICE_VERSION", "1.2.3")
    monkeypatch.setenv("OTEL_SERVICE_INSTANCE_ID", "instance-7")
    monkeypatch.setenv("PHLO_PROJECT", "demo-lakehouse")
    monkeypatch.setattr(
        "phlo_otel.provider.get_settings",
        lambda: SimpleNamespace(
            phlo_environment="prod",
            phlo_log_service_name="ignored",
            phlo_service_namespace="ignored-namespace",
            phlo_service_version="ignored-version",
            phlo_service_instance_id="ignored-instance",
            phlo_project="ignored-project",
        ),
    )

    resource_attributes = provider._build_resource_attributes()

    assert resource_attributes["service.name"] == "phlo-api"
    assert resource_attributes["service.namespace"] == "phlohouse"
    assert resource_attributes["service.version"] == "1.2.3"
    assert resource_attributes["service.instance.id"] == "instance-7"
    assert resource_attributes["deployment.environment"] == "prod"
    assert resource_attributes["phlo.package"] == "phlo-otel"
    assert resource_attributes["phlo.runtime"] == "python"
    assert resource_attributes["phlo.project"] == "demo-lakehouse"


def test_build_resource_attributes_uses_phlo_defaults(monkeypatch):
    monkeypatch.delenv("OTEL_SERVICE_NAME", raising=False)
    monkeypatch.delenv("OTEL_SERVICE_NAMESPACE", raising=False)
    monkeypatch.delenv("OTEL_SERVICE_VERSION", raising=False)
    monkeypatch.delenv("OTEL_SERVICE_INSTANCE_ID", raising=False)
    monkeypatch.delenv("PHLO_PROJECT", raising=False)
    monkeypatch.setattr(
        "phlo_otel.provider.get_settings",
        lambda: SimpleNamespace(
            phlo_environment="dev",
            phlo_log_service_name="phlo-worker",
            phlo_service_namespace="phlo",
            phlo_service_version=None,
            phlo_service_instance_id=None,
            phlo_project=None,
        ),
    )
    monkeypatch.setattr("phlo_otel.provider.socket.gethostname", lambda: "worker-host")

    resource_attributes = provider._build_resource_attributes()

    assert resource_attributes["service.name"] == "phlo-worker"
    assert resource_attributes["service.namespace"] == "phlo"
    assert resource_attributes["service.version"] == provider.INSTRUMENTATION_VERSION
    assert resource_attributes["service.instance.id"] == "worker-host"
    assert resource_attributes["deployment.environment"] == "dev"
    assert resource_attributes["phlo.project"] == "phlo-worker"


def test_build_resource_attributes_uses_phlo_observability_settings(monkeypatch):
    monkeypatch.delenv("OTEL_SERVICE_NAME", raising=False)
    monkeypatch.delenv("OTEL_SERVICE_NAMESPACE", raising=False)
    monkeypatch.delenv("OTEL_SERVICE_VERSION", raising=False)
    monkeypatch.delenv("OTEL_SERVICE_INSTANCE_ID", raising=False)
    monkeypatch.delenv("PHLO_PROJECT", raising=False)
    monkeypatch.setattr(
        "phlo_otel.provider.get_settings",
        lambda: SimpleNamespace(
            phlo_environment="staging",
            phlo_log_service_name="phlo-api",
            phlo_service_namespace="lakehouse",
            phlo_service_version="2.4.0",
            phlo_service_instance_id="api-7",
            phlo_project="acme-analytics",
        ),
    )

    resource_attributes = provider._build_resource_attributes()

    assert resource_attributes["service.name"] == "phlo-api"
    assert resource_attributes["service.namespace"] == "lakehouse"
    assert resource_attributes["service.version"] == "2.4.0"
    assert resource_attributes["service.instance.id"] == "api-7"
    assert resource_attributes["deployment.environment"] == "staging"
    assert resource_attributes["phlo.project"] == "acme-analytics"


def test_get_log_emitter_uses_cached_logger_provider(monkeypatch):
    provider._initialized = True
    fake_logger = object()
    fake_provider = SimpleNamespace(get_logger=lambda name, version: fake_logger)
    monkeypatch.setattr(provider, "_logger_provider", fake_provider)

    assert provider.get_log_emitter() is fake_logger


def test_get_log_emitter_returns_none_when_logs_disabled(monkeypatch):
    monkeypatch.setattr(provider, "_initialized", True)
    monkeypatch.setattr(provider, "_logger_provider", None)

    assert provider.get_log_emitter() is None


def test_signal_export_enabled_defaults_off_without_endpoint(monkeypatch):
    monkeypatch.delenv("OTEL_EXPORTER_OTLP_ENDPOINT", raising=False)
    monkeypatch.delenv("OTEL_EXPORTER_OTLP_TRACES_ENDPOINT", raising=False)
    monkeypatch.delenv("OTEL_TRACES_EXPORTER", raising=False)

    assert provider._traces_export_enabled() is False


def test_signal_export_enabled_uses_endpoint_when_exporter_unset(monkeypatch):
    monkeypatch.setenv("OTEL_EXPORTER_OTLP_ENDPOINT", "http://collector:4317")
    monkeypatch.delenv("OTEL_TRACES_EXPORTER", raising=False)

    assert provider._traces_export_enabled() is True


def test_shutdown_otel_resets_cached_providers(monkeypatch):
    calls: list[str] = []

    class FakeProvider:
        def __init__(self, name: str) -> None:
            self._name = name

        def shutdown(self) -> None:
            calls.append(self._name)

    monkeypatch.setattr(provider, "_initialized", True)
    monkeypatch.setattr(provider, "_logger_provider", FakeProvider("logs"))
    monkeypatch.setattr(provider, "_meter_provider", FakeProvider("metrics"))
    monkeypatch.setattr(provider, "_tracer_provider", FakeProvider("traces"))

    provider.shutdown_otel()

    assert calls == ["logs", "metrics", "traces"]
    assert provider._initialized is False
    assert provider._logger_provider is None
    assert provider._meter_provider is None
    assert provider._tracer_provider is None
